function [vi_sal, nv] = hks_detect(shape, t_ind)
%  
% [vi_sal, nv] = hks_detect(shape, t_ind)
%   input
%   shape: shape.X, shape.Y, shape.TRI (vertx and triangles)
%   t_ind: scale
%   output
%   vi_sal: index of salient points
%   nv: number of vertices
%    
% Eurographics 2012 - Tutorial on "Diffusion Geometry in Shape Analysis"
% Umberto Castellani, Alex Bronstein, Michael Bronstein
%

%

% Eigendecomposition
[W, A] = mshlp_matrix(shape);
Am = sparse([1:length(A)], [1:length(A)], A);
warning off;
[evecs evals] = eigs(W, Am, 200, -1e-5);
warning on;
evals = diag(evals);

tmin = abs(4*log(10) / evals(end));
tmax = abs(4*log(10) / evals(2));
nstep = 100;

stepsize = (log(tmax) - log(tmin)) / nstep;
logts = log(tmin):stepsize:log(tmax);
ts = exp(logts);

hks = abs( evecs(:, 2:end) ).^2 * exp( - abs(evals(2:end)) * ts);


% Compute salient point
v_sal = [];
nv=size(shape.X,1);

vr = compute_vertex_ring(shape.TRIV);

% number of rings to consider in order to determine if there is a local
% maxima
nrings = 2;

for j=2:nrings;
    for i=1:nv
        vrr{i} = unique([vr{vr{i}}]);
    end
    vr = vrr;
end

hks_vals=hks(:,t_ind);

for(i=1:nv)
    % if i is maximum (no higher values) 
    v_sal(i)=~sum(hks_vals(i)<hks_vals(vr{i}));
end
vi_sal=find(v_sal);

end

