function plotMesh(vertices,faces,options)
% plot_mesh - plot a 3D mesh.
%
%   plot_mesh(vertices,faces, options);
%
%   'options' is a structure that may contains:
%       - 'face_color' : a float specifying the color of the faces.
%       - 'face_vertex_color' : a color per vertex or face.
%       - 'paths': geodesic curves path
%  Roberto Toldo roberto.toldo@gmail.com - Vips Laboratory University of Verona '08


if nargin<2
    error('Not enough arguments.');
end
if nargin<3
    options = [];
end


% can flip to accept data in correct ordering
if size(vertices,1)==3 && size(vertices,2)~=3
    vertices = vertices';
end
if size(faces,1)==3 && size(faces,2)~=3
    faces = faces';
end

if size(faces,2)~=3 || (size(vertices,2)~=3 && size(vertices,2)~=2)
    error('face or vertex does not have correct format.');
end


if ~isfield(options, 'face_color')
    %options.face_color = 0.7;
    options.face_color = [0.5, 0.5, 0.5];
end
face_color = options.face_color;

nverts = size(vertices,1);
if isfield(options, 'face_vertex_color')
   face_vertex_color = options.face_vertex_color;
else
    face_vertex_color = repmat(face_color,nverts,1);
end

if isfield(options, 'shading')
   shading = options.shading;
else
    shading = 'interp';
end

if isfield(options, 'EdgeColor')
    EdgeColor = options.EdgeColor;
else
    EdgeColor = [0.0, 0.0, 0.0];
end

colormap('default')

% patch('vertices',vertices,'faces',faces,'FaceVertexCData',face_vertex_color, 'FaceColor','interp','LineWidth',0.01,'EdgeColor',EdgeColor);
trisurf(faces,vertices(:, 1),vertices(:, 2),vertices(:, 3),'FaceVertexCData',face_vertex_color, 'FaceColor','interp','LineWidth',0.01,'EdgeColor',EdgeColor);
hold on;

lighting gouraud;
%lighting none;
axis equal;
grid on;

if isfield(options, 'paths')
    paths = options.paths;
    for i=1:length(options.paths)
        p = paths{i};
        plot3(p(1,:), p(2,:), p(3,:),'r.','LineWidth',5);
        % Umbe: per fare la figura del path
        %plot3(p(1,1:10:end), p(2,1:10:end), p(3,1:10:end),'m.','MarkerSize',20);
        %plot3(p(1,1:10:end), p(2,1:10:end), p(3,1:10:end),'b-','LineWidth',5);
        %plot3(p(1,:), p(2,:), p(3,:),'b-','LineWidth',3);
    end
end

if isfield(options, 'mark')
    m = find(options.mark == 1);
     % Umbe: visualizzo un punto pi grosso
    %plot3(vertices(m,1), vertices(m,2), vertices(m,3),'ro','MarkerSize',10);
    plot3(vertices(m,1), vertices(m,2), vertices(m,3),'r.','MarkerSize',30);
end
