% 
% Eurographics 2012 - Tutorial on "Diffusion Geometry in Shape Analysis"
%
%  Umberto Castellani, Alex Bronstein, Michael Bronstein
%


close all
clear

% Read model as ply format
%
data = ply_read('models\Male_null.ply');
%
shape.X = data.vertex.x;
shape.Y = data.vertex.y;
shape.Z = data.vertex.z;
TRIV = zeros(length(data.face.vertex_indices),3);
for k = 1:length(data.face.vertex_indices)
   TRIV(k,:) = data.face.vertex_indices{k} + 1; 
end
shape.TRIV = TRIV;

% Show 3D model
%
figure(1)
trisurf(shape.TRIV,shape.X,shape.Y,shape.Z, ones(length(shape.X), 1)), shading interp, axis image, axis tight, lighting phong, camlight


% Eigendecomposition
%
% Compute matrices (see http://www.geomtop.org/software/meshlp.html)
%
[W, A] = mshlp_matrix(shape);
Am = sparse([1:length(A)], [1:length(A)], A);
warning off;
% Compute eigendecomposition
[evecs evals] = eigs(W, Am, 200, -1e-5);
warning on;
evals = abs(diag(evals));

abssc=0;

if (abssc)
    % Linear scales
    %
    % small scales
    %ts = 1:4:400;
    %
    % larger scale
    ts = 1:200:20000;
else
    % Nonlinear-scales (euristic from
    % http://www.geomtop.org/paper_ppt/multiscale_htker.pdf)
    %
    tmin = abs(4*log(10) / evals(end));
    tmax = abs(4*log(10) / evals(2));
    nstep = 100;
    stepsize = (log(tmax) - log(tmin)) / nstep;
    logts = log(tmin):stepsize:log(tmax);
    ts = exp(logts);
end



TT  = repmat(ts(:)', [length(evals) 1]);
LL  = repmat(evals(:) , [1 length(ts)]);

% Show diffusion distance
%
for i=1:100:length(evecs)
    v_i=i;
    %
    fi_i=evecs(v_i,:);
    %

    FI_I= repmat(fi_i, [length(evecs), 1]);
    %
    VV  = (evecs-FI_I).^2;

    DD_i = max(VV*(exp(-LL.*TT)), 0);

    disp(['We are analysing vertex ' num2str(i)]);
    for t=1:10:length(ts);
    figure(4)
    hold off
    plot3(shape.X(v_i), shape.Y(v_i), shape.Z(v_i), 'r.', 'Markersize', 30);
    hold on
    trisurf(shape.TRIV,shape.X,shape.Y,shape.Z,DD_i(:,t)), shading interp, axis image, axis tight, lighting phong, camlight
    colorbar
    end
    disp('Now a new point is considered');
    input('...');
end