% 
% Eurographics 2012 - Tutorial on "Diffusion Geometry in Shape Analysis"
%
%  Umberto Castellani, Alex Bronstein, Michael Bronstein
%


clc
close all
clear

%###########################################################################
% Evaluate centaur at high scale
disp('Centaur');
data = ply_read('models\Centaur_null.ply');
% Deal with meshes
shape.X = data.vertex.x;
shape.Y = data.vertex.y;
shape.Z = data.vertex.z;
TRIV = zeros(length(data.face.vertex_indices),3);
for k = 1:length(data.face.vertex_indices)
   TRIV(k,:) = data.face.vertex_indices{k} + 1; 
end
%
shape.TRIV = TRIV;
figure
trisurf(shape.TRIV,shape.X,shape.Y,shape.Z, ones(length(shape.X), 1)), shading interp, axis image, axis tight, lighting phong, camlight
%
t_ind = 50;
%
[vi_sal, nv]=hks_detect(shape, t_ind);
%
saliencyMarks=false(nv,1);
saliencyMarks(vi_sal)=true;
options.mark = saliencyMarks;
figure
plotMesh([shape.X, shape.Y, shape.Z],shape.TRIV,options); shading interp, axis image, axis tight, lighting phong, camlight
%
nsp=length(vi_sal);
%
disp(['Number of salient points: ' num2str(nsp)]);
disp('');
input('...');
disp('');
%
%###########################################################################
% Evaluate centaur at medium scale
%
t_ind = 10;
[vi_sal, nv]=hks_detect(shape, t_ind);
%
saliencyMarks=false(nv,1);
saliencyMarks(vi_sal)=true;
options.mark = saliencyMarks;
figure
plotMesh([shape.X, shape.Y, shape.Z],shape.TRIV,options); shading interp, axis image, axis tight, lighting phong, camlight
%
nsp=length(vi_sal);
%
disp(['Number of salient points: ' num2str(nsp)]);
disp('');
input('...');
disp('');

%###########################################################################
% Evaluate centaur at small scale
t_ind = 1;
[vi_sal, nv]=hks_detect(shape, t_ind);
%
saliencyMarks=false(nv,1);
saliencyMarks(vi_sal)=true;
options.mark = saliencyMarks;
figure
plotMesh([shape.X, shape.Y, shape.Z],shape.TRIV,options); shading interp, axis image, axis tight, lighting phong, camlight
%
nsp=length(vi_sal);
%
disp(['Number of salient points: ' num2str(nsp)]);
disp('');
input('...');
disp('');

%###########################################################################
% Evaluate Null shape Male at high scale
disp('Male null shape');
data = ply_read('models\Male_null.ply');
% Deal with meshes
shape.X = data.vertex.x;
shape.Y = data.vertex.y;
shape.Z = data.vertex.z;
TRIV = zeros(length(data.face.vertex_indices),3);
for k = 1:length(data.face.vertex_indices)
   TRIV(k,:) = data.face.vertex_indices{k} + 1; 
end
%
shape.TRIV = TRIV;
figure
trisurf(shape.TRIV,shape.X,shape.Y,shape.Z, ones(length(shape.X), 1)), shading interp, axis image, axis tight, lighting phong, camlight
%
t_ind = 50;
%
[vi_sal, nv]=hks_detect(shape, t_ind);
%
saliencyMarks=false(nv,1);
saliencyMarks(vi_sal)=true;
options.mark = saliencyMarks;
figure
plotMesh([shape.X, shape.Y, shape.Z],shape.TRIV,options); shading interp, axis image, axis tight, lighting phong, camlight
%
nsp=length(vi_sal);
%
disp(['Number of salient points: ' num2str(nsp)]);
disp('');
input('...');
disp('');
%
%###########################################################################
% Evaluate Null shape male at medium scale
%
t_ind = 10;
[vi_sal, nv]=hks_detect(shape, t_ind);
%
saliencyMarks=false(nv,1);
saliencyMarks(vi_sal)=true;
options.mark = saliencyMarks;
figure
plotMesh([shape.X, shape.Y, shape.Z],shape.TRIV,options); shading interp, axis image, axis tight, lighting phong, camlight
%
nsp=length(vi_sal);
%
disp(['Number of salient points: ' num2str(nsp)]);
disp('');
input('...');
disp('');

%###########################################################################
% Evaluate Null shape Male at small scale
t_ind = 1;
[vi_sal, nv]=hks_detect(shape, t_ind);
%
saliencyMarks=false(nv,1);
saliencyMarks(vi_sal)=true;
options.mark = saliencyMarks;
figure
plotMesh([shape.X, shape.Y, shape.Z],shape.TRIV,options); shading interp, axis image, axis tight, lighting phong, camlight
%
nsp=length(vi_sal);
%
disp(['Number of salient points: ' num2str(nsp)]);
disp('');


